@@ -2,7 +2,7 @@
 using namespace Rcpp;
 
 //[[Rcpp::export]]
-List maxmind_(std::vector < std::string > ips, const char* file, std::vector < std::string > fields){
+List maxmind_(CharacterVector ips, const char* file, std::vector < std::string > fields){
   maxmind_bindings bind_inst;
   return bind_inst.call_maxmind(ips, file, fields);
 }
